# -*- coding: utf-8 -*-
"""Opinionated CDK construct to create SNS topic.

Security parameters are set by default
"""
from constructs import Construct
import aws_cdk.aws_sns as sns
from aws_cdk import aws_kms as kms
from aws_cdk import aws_iam as iam

from typing import Union


class SNSTopic(Construct):
    """CDK SNS topic construct."""

    # pylint: disable=W0235
    # pylint: disable=W0622
    def __init__(self, scope: Construct, id: str):
        """

        :param scope:
        :param id:
        """
        super().__init__(scope, id)

    def create_sns_topic(self, topic_name: str, master_key: Union[kms.IKey, None]) -> sns.Topic:
        """Create SNS topic.

        :param topic_name: The name of SNS topic
        :param master_key: The KMS key to encrypt messages going through sns topic
        :return: The CDK object for SNS topic
        """
        return sns.Topic(self, id=topic_name, topic_name=topic_name, master_key=master_key)

    @staticmethod
    def create_sns_topic_policy(sns_topic: sns.ITopic):
        """Create topic policy that denies unencrypted access."""
        sns_topic.add_to_resource_policy(
            iam.PolicyStatement(
                sid="AllowPublishThroughSSLOnly",
                actions=["sns:Publish"],
                effect=iam.Effect.DENY,
                resources=[sns_topic.topic_arn],
                conditions={
                    "Bool": {"aws:SecureTransport": "false"},
                },
                principals=[iam.AnyPrincipal()],
            )
        )



# -*- coding: utf-8 -*-
"""Opinionated CDK construct to create S3 bucket.

Security parameters are set by default
"""
from constructs import Construct
import aws_cdk as cdk
import aws_cdk.aws_s3 as s3
from aws_cdk import aws_kms as kms

from typing import Union


class S3Bucket(Construct):
    """Create opinionated S3 bucket including restricted public access,
    enforced encryption in transit as well as in rest, versioning and lifecycle
    rules."""

    # pylint: disable=W0235
    # pylint: disable=W0622
    def __init__(self, scope: Construct, id: str):
        """

        :param scope:
        :param id:
        """
        super().__init__(scope, id)

    def create_bucket(
        self,
        bucket_name: str,
        kms_key: kms.IKey,
        server_access_logs_bucket: Union[None, s3.IBucket] = None,
        enforce_ssl: bool = True,
        **kwargs,
    ) -> s3.Bucket:
        """Create S3 bucket.

        :param kms_key: The kms to be used.
        :param bucket_name: The name of S3 bucket.
        :param enforce_ssl: Bool value if SSL should be enforced.
        :param server_access_logs_bucket: The CDK object for S3 bucket.
        :param kwargs:
         event_bridge_enabled: bool - set to True if s3 events should be sent to event bridge
         server_access_logs_prefix: str - in which prefix logs should be stored
        :return: The S3 bucket CDK object
        """

        return s3.Bucket(
            self,
            id=bucket_name,
            auto_delete_objects=True,
            block_public_access=s3.BlockPublicAccess(
                block_public_acls=True, block_public_policy=True, ignore_public_acls=True, restrict_public_buckets=True
            ),
            bucket_name=bucket_name,
            encryption=s3.BucketEncryption.KMS,
            encryption_key=kms_key,
            enforce_ssl=enforce_ssl,
            event_bridge_enabled=True if kwargs.get("event_bridge_enabled") else False,
            lifecycle_rules=[
                s3.LifecycleRule(enabled=True, noncurrent_version_expiration=cdk.Duration.days(amount=1)),
                s3.LifecycleRule(
                    enabled=True,
                    expired_object_delete_marker=True,
                    abort_incomplete_multipart_upload_after=cdk.Duration.days(7),
                ),
            ],
            public_read_access=False,
            removal_policy=cdk.RemovalPolicy.DESTROY,
            server_access_logs_prefix=kwargs.get("server_access_logs_prefix"),
            server_access_logs_bucket=server_access_logs_bucket,
            versioned=True,
        )



# -*- coding: utf-8 -*-
"""Opinionated CDK construct to create AWS Lambda function.

Security parameters are set by default
"""
from constructs import Construct
import aws_cdk as cdk
import aws_cdk.aws_iam as iam
import aws_cdk.aws_lambda as lmb
from aws_cdk import aws_signer as signer
from aws_cdk import aws_logs as logs

from typing import Dict, Union


class AWSPythonLambdaFunction(Construct):
    """Create Lambda function and supported objects like lambda layer, signing
    profile, IAM role and policies."""

    # pylint: disable=W0235
    # pylint: disable=W0622
    def __init__(self, scope: Construct, id: str):
        """

        :param scope:
        :param id:
        """
        super().__init__(scope, id)

    def signing_profile(self, signing_profile_name: str) -> signer.ISigningProfile:
        """Create code signing profile using AWS Signer to be used in signing
        config.

        :param signing_profile_name: The name of signing profile
        :return: AWS Signer signing profile
        """

        return signer.SigningProfile(
            self,
            "signing-profile",
            platform=signer.Platform.AWS_LAMBDA_SHA384_ECDSA,
            signing_profile_name=signing_profile_name,
        )

    def signing_config(self, profile: signer.ISigningProfile) -> lmb.ICodeSigningConfig:
        """Create code signing config using AWS Lambda to sign lambda code
        using profile created by AWS Signer.

        :param profile: AWS Signer profile
        :return: AWS Lambda code signing config
        """

        return lmb.CodeSigningConfig(self, "conde-signing-config", signing_profiles=[profile])

    def create_lambda_layer(self, code_path: str, construct_id: str = "supporting_libraries") -> lmb.LayerVersion:
        """Create lambda layer.

        :param props: The dictionary which contain configuration values.
        :param code_path: path which contain lambda layer directory
        :param construct_id: construct id
        :return: Lambda layer
        """
        return lmb.LayerVersion(
            self,
            id=construct_id,
            code=lmb.Code.from_asset(code_path),
            compatible_runtimes=[lmb.Runtime.PYTHON_3_9],
        )

    # pylint: disable=R0913
    def create_lambda_function(
        self,
        code_path: str,
        env: cdk.Environment,
        env_variables: Dict,
        function_name: str,
        layer: lmb.ILayerVersion,
        reserved_concurrent_executions: Union[None, int],
        timeout: int,
        architecture: lmb.Architecture = lmb.Architecture.ARM_64,
        memory_size: int = 256,
        handler: str = "handler.handler",
        signing_config: Union[lmb.ICodeSigningConfig, None] = None,
        **kwargs,
    ) -> lmb.Function:
        """Create lambda function.

        :param architecture: Lambda CPU architecture, default ARM_64
        :param code_path: path which contain lambda function directory
        :param env: The CDK Environment object which consist region and aws account id
        :param env_variables: Dictionary which contain additional lambda env variables
        :param function_name: The name of lambda function
        :param handler: Lambda handler, default handler.handler
        :param layer: Contains lambda layer object
        :param memory_size: Lambda memory size, default 256MB
        :param reserved_concurrent_executions: The number of max concurrent lambda executions
        :param signing_config: Contains signing config for lambda, default None
        :param timeout: Lambda timeout in seconds,
        :param kwargs:
            * security_groups - list of ec2.SecurityGroup, the vpc security groups to be assigned to lambda function
            * vpc - the ec2.IVpc object, lambda will be assigned to this VPC
            * vpc_subnets - ec2.SubnetSelection, in which subnets lambda will operate
        :return: Lambda function
        """
        lambda_environment_default_variables = {
            "CLOUDWATCH_SAMPLING_RATE": "1",
            "REGION_NAME": env.region,
        }

        return lmb.Function(
            self,
            architecture=architecture,
            code=lmb.Code.from_asset(code_path),
            code_signing_config=signing_config,
            environment={**lambda_environment_default_variables, **env_variables},
            function_name=function_name,
            handler=handler,
            id=function_name,
            initial_policy=[
                iam.PolicyStatement(effect=iam.Effect.ALLOW, actions=["logs:CreateLogGroup"], resources=["*"]),
                iam.PolicyStatement(
                    effect=iam.Effect.ALLOW,
                    actions=["logs:CreateLogStream", "logs:PutLogEvents"],
                    resources=["arn:aws:logs:*:*:log-group:/aws/lambda-insights:*"],
                ),
            ],
            layers=[layer],
            log_retention=logs.RetentionDays.ONE_WEEK,
            memory_size=memory_size,
            profiling=True,
            reserved_concurrent_executions=reserved_concurrent_executions,
            runtime=lmb.Runtime.PYTHON_3_9,
            security_groups=kwargs.get("security_groups"),
            timeout=cdk.Duration.seconds(timeout),
            tracing=lmb.Tracing.ACTIVE,
            vpc=kwargs.get("vpc"),
            vpc_subnets=kwargs.get("vpc_subnets"),
        )



# -*- coding: utf-8 -*-
"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
"""

from setuptools import setup, find_packages

setup(
    name="cdk-opinionated-constructs",
    version="1.5.0",
    description="AWS CDK constructs come without added security configurations.",
    long_description="Very rarely this is validated during the CI pipeline via tools like CDK-NAG. The idea behind this project is to create secured constructs from the start.",
    license="MIT",
    package_dir={"": "."},
    packages=find_packages(where="."),
    install_requires=[
        "aws-cdk-lib>=2.50.0",
        "constructs>=10.0.0,<11.0.0",
    ],
    python_requires=">=3.9",
)
